/*
 * File:   system_init.c
 * Author: yts
 *
 * Created on 2017/10/04, 9:05
 */

// DSPIC33FJ128GP802 Configuration Bit Settings

// 'C' source line config statements


// FBS
#pragma config BWRP = WRPROTECT_OFF     // Boot Segment Write Protect (Boot Segment may be written)
#pragma config BSS = NO_FLASH           // Boot Segment Program Flash Code Protection (No Boot program Flash segment)
#pragma config RBS = NO_RAM             // Boot Segment RAM Protection (No Boot RAM)

// FSS
#pragma config SWRP = WRPROTECT_OFF     // Secure Segment Program Write Protect (Secure segment may be written)
#pragma config SSS = NO_FLASH           // Secure Segment Program Flash Code Protection (No Secure Segment)
#pragma config RSS = NO_RAM             // Secure Segment Data RAM Protection (No Secure RAM)

// FGS
#pragma config GWRP = OFF               // General Code Segment Write Protect (User program memory is not write-protected)
#pragma config GSS = OFF                // General Segment Code Protection (User program memory is not code-protected)

// FOSCSEL
#pragma config FNOSC = PRIPLL           // Oscillator Mode (Primary Oscillator (XT, HS, EC) w/ PLL)
#pragma config IESO = ON                // Internal External Switch Over Mode (Start-up device with FRC, then automatically switch to user-selected oscillator source when ready)

// FOSC
#pragma config POSCMD = XT              // Primary Oscillator Source (XT Oscillator Mode)
#pragma config OSCIOFNC = OFF           // OSC2 Pin Function (OSC2 pin has clock out function)
#pragma config IOL1WAY = ON             // Peripheral Pin Select Configuration (Allow Only One Re-configuration)
#pragma config FCKSM = CSECME           // Clock Switching and Monitor (Both Clock Switching and Fail-Safe Clock Monitor are enabled)

// FWDT
#pragma config WDTPOST = PS32768        // Watchdog Timer Postscaler (1:32,768)
#pragma config WDTPRE = PR128           // WDT Prescaler (1:128)
#pragma config WINDIS = OFF             // Watchdog Timer Window (Watchdog Timer in Non-Window mode)
#pragma config FWDTEN = OFF             // Watchdog Timer Enable (Watchdog timer enabled/disabled by user software)

// FPOR
#pragma config FPWRT = PWR128           // POR Timer Value (128ms)
#pragma config ALTI2C = OFF             // Alternate I2C  pins (I2C mapped to SDA1/SCL1 pins)

// FICD
#pragma config ICS = PGD1               // Comm Channel Select (Communicate on PGC1/EMUC1 and PGD1/EMUD1)
#pragma config JTAGEN = OFF             // JTAG Port Enable (JTAG is Disabled)

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

#include <xc.h>
#include "dsp.h"
#include "adcdacDrv.h"

fractional Buffer __attribute__((space(dma)));  

/*=============================================================================
initAdc() is used to configure A/D to convert channel 9 on Timer event. 
It generates event to DMA on every sample/convert sequence.  
=============================================================================*/
void initAdc(void)
{
    AD1CON1bits.ADSIDL = 0; //Continue in Idle mode
    AD1CON1bits.ADDMABM = 1; // DMA buffers are built in conversion order mode
    AD1CON1bits.AD12B = 1;	 // 12-bit ADC operation
    AD1CON1bits.FORM  = 3;	 // Data Output Format: Signed Fraction (Q15 format)
    AD1CON1bits.SSRC  = 2;	 // Sample Clock Source: GP Timer starts conversion
    AD1CON1bits.ASAM  = 1;	 // ADC Sample Control: Sampling begins immediately after conversion


    AD1CON2bits.CHPS = 0;	 // Converts CH0
    AD1CON2bits.SMPI    = 0; // SMPI must be 0
    
 	 // ADC Conversion Time for 12-bit Tc=14*Tad = 1.4us   
    AD1CON3bits.ADRC = 0;	 // ADC Clock is derived from Systems Clock
    AD1CON3bits.ADCS = 3;	 // ADC Conversion Clock Tad=Tcy*(ADCS+1)= (1/40M)*4 = 100ns

    //AD1CHS0: A/D Input Select Register
    AD1CHS0bits.CH0SA = 9;	 // MUXA +ve input selection (AN9) for CH0
    AD1CHS0bits.CH0NA = 0;	 // MUXA -ve input selection (Vref-) for CH0

    //AD1PCFGH/AD1PCFGL: Port Configuration Register
    AD1PCFGL           = 0xFFFF;
    AD1PCFGLbits.PCFG9 = 0;     // AN9 as Analog Input

    
    IFS0bits.AD1IF   = 0;	// Clear the A/D interrupt flag bit
    IEC0bits.AD1IE   = 0;	// Do Not Enable A/D interrupt
    AD1CON1bits.ADON = 1;	// Turn on the A/D converter	
}

/*=============================================================================
initDac() is used to configure D/A. 
=============================================================================*/
void initDac(void)
{
    /* Initiate DAC Clock */
    ACLKCONbits.SELACLK  = 0;  // PLL as Clock Source
    ACLKCONbits.AOSCMD   = 0;  // Auxiliary Oscillator Disabled
    ACLKCONbits.APSTSCLR = 7;  // Fvco/1 = 160 MHz/1 = 160 MHz
    ACLKCONbits.ASRCSEL  = 0;  // No effect

    // Sampling Rate Fs = DACCLK/256 = 104.1 kHz
    DAC1CONbits.DACSIDL = 0;    //Continue in Idle mode
    DAC1CONbits.AMPON   = 0;   // Analog Output Amplifier is enabled during Sleep Mode/Stop-in Idle mode
    DAC1CONbits.FORM    = 1;   // Data Format is signed integer
    DAC1CONbits.DACFDIV = 5;   // DACCLK = ACLK/(DACFDIV + 1): 160 MHz/6 = 26.7 MHz

    DAC1STATbits.ROEN = 1;	// Right Channel DAC Output Enabled
    DAC1DFLT          = 0x0000;	// DAC Default value is the midpoint
    
    DAC1CONbits.DACEN   = 1;   // DAC1 Module Enabled
}

/*=======================================================================================  
Timer 3 is setup to time-out every Ts secs. As a result, the module 
will stop sampling and trigger a conversion on every Timer3 time-out Ts. 
At that time, the conversion process starts and completes Tc=12*Tad periods later.
When the conversion completes, the module starts sampling again. However, since Timer3 
is already on and counting, about (Ts-Tc)us later, Timer3 will expire again and trigger 
next conversion. 
=======================================================================================*/
void initTmr3() 
{
    TMR3          = 0x0000;   // Clear TMR3
    PR3           = SAMPPRD;  // Load period value in PR3
    IFS0bits.T3IF = 0;	   // Clear Timer 3 Interrupt Flag
    IEC0bits.T3IE = 0;	   // Clear Timer 3 interrupt enable bit
	
    T3CONbits.TON = 1;	   // Enable Timer 3
}

/*=============================================================================  
DMA0 configuration
 Direction: Read from peripheral address 0-x300 (ADC1BUF0) and write to DMA RAM 
 AMODE: Register indirect with post increment
 MODE: Continuous, Ping-Pong Mode
 IRQ: ADC Interrupt
 ADC stores results stored alternatively between BufferA[] and BufferB[]
=============================================================================*/
void initDma0(void)
{
    DMA0CONbits.AMODE = 1;      // Configure DMA for Register indirect without post increment
    DMA0CONbits.MODE  = 0;      // Configure DMA for Continuous Ping-Pong mode disable

    DMA0PAD = (int)&ADC1BUF0;   // Peripheral Address Register: ADC buffer
    DMA0CNT = (NUMSAMP-1);      // DMA Transfer Count is (NUMSAMP-1)
	
    DMA0REQ =  13;	            // ADC interrupt selected for DMA channel IRQ
	
    DMA0STA = __builtin_dmaoffset(&Buffer); // DMA RAM Start Address A

    IFS0bits.DMA0IF  = 0;       // Clear the DMA interrupt flag bit
    IEC0bits.DMA0IE  = 1;       // Set the DMA interrupt enable bit
	
    DMA0CONbits.CHEN = 1;       // Enable DMA channel
}

void SYS_Initialize ( void )
{

    // Clock settings for 40MHz
    // Fosc= Fin * M / (N1 * N2), Fcy = Fosc / 2
    // Fosc= 10 MHz * 32 / (2 * 2) = 80 MHz, Fcy = Fosc/2 = 40MHz
    CLKDIVbits.PLLPRE = 0;      	// N2=2 (1,2,4,8,16,32,64,256) 0.8-8 MHz
    PLLFBD = 30;               		// M=32 (2-33) 100-200 MHz
    CLKDIVbits.PLLPOST = 0;     	// N1=2 (2,4,8) 12.5-80 MHz

    // wait PLL lock
    while(OSCCONbits.LOCK !=1 );
	
    initAdc();    // Initialize the A/D converter to convert Channel 4
    initDac();    // Initialize the D/A converter
    initDma0();   // Initialize the DMA controller to buffer ADC data in conversion order
    initTmr3();   // Initialize the Timer to generate sampling event for ADC           

}

/*=============================================================================
_DMA0Interrupt(): ISR name is chosen from the device linker script.
=============================================================================*/
extern fractional bpfCoeffs[NUM_TAPS];
extern fractional FilterOut;
extern FIRStruct BPF;

void __attribute__((interrupt, no_auto_psv)) _DMA0Interrupt(void)
{
    //  Perform FIR Band Pass filtering 
    FIR(1, &FilterOut, &Buffer, &BPF);
    while(DAC1STATbits.REMPTY != 1);     // Wait for D/A conversion

    DAC1RDAT = FilterOut;    // Load the DAC buffer with data
               
    IFS0bits.DMA0IF = 0;    // Clear the DMA0 Interrupt Flag
}

/*=============================================================================
_DAC1RInterrupt(): ISR name is chosen from the device linker script.
=============================================================================*/
void __attribute__((interrupt, no_auto_psv)) _DAC1RInterrupt(void)
{
    IFS4bits.DAC1RIF = 0; // Clear Right Channel Interrupt Flag
}
